/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "CDisplay.hpp"
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <set>
#include <iostream>

ALLEGRO_DISPLAY* CDisplay::_display    = nullptr; 
ALLEGRO_FONT*    CDisplay:: main_font    = nullptr;
IDrawable*       CDisplay::_main_widget = nullptr;

bool             CDisplay::_main_widget_changed = false;

void CDisplay::Init(){
    al_init();
    al_install_mouse();
    al_install_keyboard();
    al_init_primitives_addon();
    al_init_font_addon();
    al_init_ttf_addon();
    al_init_image_addon();
   
    _display = al_create_display(800,600);

    main_font = al_load_ttf_font("Ubuntu-R.ttf",12,0);
    
    al_clear_to_color(al_map_rgb(0,0,0));
    al_flip_display();
}

void CDisplay::_Redraw(){
    //al_clear_to_color(al_map_rgb(0,0,0));
    if(_main_widget != nullptr){
        _main_widget->Draw( SDrawArea(0,0,al_get_display_width(_display),al_get_display_height(_display)) );
    }
    al_flip_display();
}

void CDisplay::MainLoop(){
   
    ALLEGRO_EVENT_QUEUE *event_queue = nullptr;
    event_queue = al_create_event_queue(); 
    al_register_event_source(event_queue, al_get_mouse_event_source());
    al_register_event_source(event_queue, al_get_keyboard_event_source());
    al_register_event_source(event_queue, al_get_display_event_source(_display));

    while(1)
    {
        ALLEGRO_EVENT ev;
        ALLEGRO_TIMEOUT timeout;
        al_init_timeout(&timeout, 0.06);

        bool get_event = al_wait_for_event_until(event_queue, &ev, &timeout);

        if(get_event){
            if (ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {
                break;
            }else if(ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_DOWN){
                int x = ev.mouse.x;
                int y = ev.mouse.y;
                if(_main_widget != nullptr){
                    _main_widget->OnClicked(SCoordinates(x,y));
                }
            }else if(ev. type == ALLEGRO_EVENT_MOUSE_AXES){
                int x = ev.mouse.x;
                int y = ev.mouse.y;
                if(_main_widget != nullptr){
                    _main_widget->OnMouseMovedOver(SCoordinates(x,y));
                }
	    }else if(ev.type == ALLEGRO_EVENT_MOUSE_ENTER_DISPLAY){
                int x = ev.mouse.x;
                int y = ev.mouse.y;
                if(_main_widget != nullptr){
		    _main_widget->OnMouseIn();
                    _main_widget->OnMouseMovedOver(SCoordinates(x,y));
                }
	    }else if(ev.type == ALLEGRO_EVENT_MOUSE_LEAVE_DISPLAY){
                if(_main_widget != nullptr){
		    _main_widget->OnMouseOut();
                }
	    }else if(ev.type == ALLEGRO_EVENT_KEY_CHAR){
                if(_main_widget != nullptr){
		    _main_widget->OnKeypress(SKeypressData(ev.keyboard.keycode,ev.keyboard.modifiers,ev.keyboard.unichar));
                }
	    }
        }
        
        if(_main_widget_changed){
            _Redraw();
            _main_widget_changed = false;
        }else if(_main_widget != nullptr && _main_widget->_needs_redrawing){
            _Redraw();
        }

    }

    al_destroy_event_queue(event_queue);
}

void CDisplay::Cleanup(){
    al_destroy_font(main_font);
    al_destroy_display(_display);
}

void CDisplay::SetWindowSizeSmall(){
    al_resize_display(_display,800,600); //SVGA
    _Redraw();
}
void CDisplay::SetWindowSizeMedium(){
    al_resize_display(_display,1024,768); //XGA
    _Redraw();
}
void CDisplay::SetWindowSizeLarge(){
    al_resize_display(_display,1280,800); //WXGA
    _Redraw();
}
void CDisplay::SetMainWidget(IDrawable& v){
    _main_widget = &v;
    _main_widget_changed = true;
}
