/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "CMainMenu.hpp"
#include "CDisplay.hpp"
#include <cstdlib>

CMainMenu::CMainMenu():
	_w_prop_1(0.0 , 1.0, 0.0, 1.0),
	_w_label_disclaimer("Millennium, Millennium Duel and libMillennium are in no way affiliated with the Yu-Gi-Oh! Trading Card Game nor with KONAMI(R)."),
	_w_prop_2(0.75, 0.55, 0.5, 0.5, 0.3, 0.5),
	_w_button_join("Join a game"),
	_w_button_host("Host a game"),
	_w_button_options("Options"),
	_w_button_about("About"),
	_w_button_quit("Quit"),
	_w_expanding_text(0.5,1.0,0.5,1.1),
	_w_hint_label(""),
	_w_bg_image("../data/bg-placeholder.png")
{
	Add(_w_prop_1,1);
	Add(_w_prop_2,0);
	Add(_w_bg_image,2);
	_w_prop_1.Add(&_w_label_disclaimer);
	_w_prop_2.Add(&_w_vbox_main);
	_w_expanding_text.Add(&_w_hint_label);
	
	_w_vbox_main.SetSpacing(10);
	_w_vbox_main.SetMargin(16);
	_w_vbox_main.SetBackgroundColor(al_map_rgba(150,150,150,10));
	_w_vbox_main.Add(_w_button_join,0);
	_w_vbox_main.Add(_w_button_host,1);
	_w_vbox_main.Add(_w_button_options,2);
	_w_vbox_main.Add(_w_button_about,3);
	_w_vbox_main.Add(_w_expanding_text,4,BOX_EXPAND);
	_w_vbox_main.Add(_w_button_quit,5);
	
	_w_label_disclaimer.SetTextColor(al_map_rgb(255,255,255));
	
	_w_button_join.SetForegroundColor(al_map_rgb(100,130,100));
	_w_button_host.SetForegroundColor(al_map_rgb(100,100,100));
	_w_button_options.SetForegroundColor(al_map_rgb(100,100,100));
	_w_button_about.SetForegroundColor(al_map_rgb(100,100,100));
	_w_button_quit.SetForegroundColor(al_map_rgb(130,100,100));
	
	_w_button_quit.event_activated.Subscribe(this,&CMainMenu::_ButtonQuitClicked);
	_w_button_about.event_activated.Subscribe(this,&CMainMenu::_ButtonAboutClicked);
	_w_button_options.event_activated.Subscribe(this,&CMainMenu::_ButtonOptionsClicked);
	
	_w_button_join.event_mouse_in.Subscribe(this,&CMainMenu::_ButtonJoinMouseIn);
	_w_button_host.event_mouse_in.Subscribe(this,&CMainMenu::_ButtonHostMouseIn);
	_w_button_options.event_mouse_in.Subscribe(this,&CMainMenu::_ButtonOptionsMouseIn);
	_w_button_about.event_mouse_in.Subscribe(this,&CMainMenu::_ButtonAboutMouseIn);
	_w_button_quit.event_mouse_in.Subscribe(this,&CMainMenu::_ButtonQuitMouseIn);
	_w_button_join.event_mouse_out.Subscribe(this,&CMainMenu::_ClearHintText);
	_w_button_host.event_mouse_out.Subscribe(this,&CMainMenu::_ClearHintText);
	_w_button_options.event_mouse_out.Subscribe(this,&CMainMenu::_ClearHintText);
	_w_button_about.event_mouse_out.Subscribe(this,&CMainMenu::_ClearHintText);
	_w_button_quit.event_mouse_out.Subscribe(this,&CMainMenu::_ClearHintText);
}

void CMainMenu::_ButtonQuitClicked(){
	CDisplay::Cleanup();
	exit(0);
}

void CMainMenu::_ButtonAboutClicked(){ event_about_clicked.Happen();}
void CMainMenu::_ButtonOptionsClicked(){ event_options_clicked.Happen();}

void CMainMenu::_ButtonJoinMouseIn(){ _w_hint_label.SetText("Join a remote duel"); }
void CMainMenu::_ButtonHostMouseIn(){ _w_hint_label.SetText("Host a new duel"); }
void CMainMenu::_ButtonOptionsMouseIn(){ _w_hint_label.SetText("Configure the game"); }
void CMainMenu::_ButtonAboutMouseIn(){ _w_hint_label.SetText("Display game info & authors"); }
void CMainMenu::_ButtonQuitMouseIn(){ _w_hint_label.SetText("Close the game"); }
void CMainMenu::_ClearHintText(){ _w_hint_label.SetText(""); }
