#ifndef __CEVENT_HPP__
#define __CEVENT_HPP__

/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include <list>
#include <functional>

template <typename...>
class CEvent;

template <typename T>
class CEvent<T>{
private:
    std::list< std::function<void(T)> > subscribers;
    std::list< std::function<void() > > deaf_subscribers;
public:
    void Subscribe( std::function<void(T)> f ) {
	subscribers.push_back(f);
    };
    void Subscribe( std::function<void()> f ) {
	deaf_subscribers.push_back(f);
    };
    template<class C>
    void Subscribe( C* class_ptr, void (C::*member_ptr)(T)){
	subscribers.push_back( std::bind(member_ptr,class_ptr,std::placeholders::_1) );
    }
    template<class C>
    void Subscribe( C* class_ptr, void (C::*member_ptr)()){
	deaf_subscribers.push_back( std::bind(member_ptr,class_ptr) );
    }
    void Happen(T t){
	for(auto f : subscribers) f(t);
	for(auto f : deaf_subscribers) f();
    }
};

template <>
class CEvent<>{
private:
    std::list< std::function<void()> > subscribers;
public:
    void Subscribe( std::function<void()> f ) {
	subscribers.push_back(f);
    };
    template<class C>
    void Subscribe( C* class_ptr, void (C::*member_ptr)()){
	subscribers.push_back( std::bind(member_ptr,class_ptr) );
    }
    void Happen(){
	for(auto f : subscribers) f();
    }
};
#endif //__CEVENT_HPP__
