/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "IBox.hpp"

IBox::IBox() :
    _spacing(0),
    _current_width(-1),
    _current_height(-1) {}

void IBox::SetSpacing(int s){
    _spacing = s;
}

void IBox::_RecalculateSizes(int total, int (IDrawable::*size_func)()){
    if(children.size() == 0) return;

    int shrinked_sum = 0;
    int expand_count = 0;
    
    for(auto &i : children){
        if(!i.is_visible) continue; //an unvisible widget. skip it...
        
        if(i.mode == BOX_SHRINK){
            int h = ((i.item).*size_func)(); //use the size_func to get minimal width/height
            i.size = h;
            shrinked_sum += h;
        }else if(i.mode == BOX_EXPAND){
            expand_count++;
        }
        shrinked_sum += _spacing;
    }
    if (shrinked_sum > 0) shrinked_sum -= _spacing;
    
    if(expand_count > 0){
        int rest = total - shrinked_sum;
        int expand_size = rest / expand_count;
        int last_expand_size = rest - (expand_size * expand_count) + expand_size;
        
        if(expand_size < 0) expand_size = 0;
        if(last_expand_size < 0) last_expand_size = 0;
        
        for(auto &i : children){
            if(!i.is_visible) continue; //an unvisible widget. skip it...
            
            if(i.mode == BOX_EXPAND){
                if(expand_count != 1){ //this is not the last expanding item
                    i.size = expand_size;
                }else{
                    i.size = last_expand_size;
                }
                expand_count--;
            }
        }
    }
}
