/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "IOrderedContainer.hpp"
#include <iostream>

IOrderedContainer::IOrderedContainer() {}

void IOrderedContainer::Add(IDrawable& item, int position, BoxPackingMode mode, bool is_visible){
    if(item.parent != nullptr){
        std::cerr << "Unable to add item to OrderedContainer: item already has a parent" << std::endl;
        return;
    }
    ContainerEntry ce(item);
    ce.order = position;
    ce.mode = mode;
    ce.is_visible = is_visible;
    children.insert(ce);
    item.parent = this;
    
    SetNeedsRedrawing();
}

void IOrderedContainer::SetVisibility(IDrawable& item, bool is_visible)
{
    for(auto &i : children)
        if(&i.item == &item)
        {
            i.is_visible = is_visible;
            break;
        }
        
    SetNeedsRedrawing();
}
