#ifndef __IORDEREDCONTAINER_HPP__
#define __IORDEREDCONTAINER_HPP__

/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "IContainer.hpp"
#include <set>

enum BoxPackingMode{
    BOX_SHRINK = 0,
    BOX_EXPAND = 1
};

class IOrderedContainer : public IContainer{
public:
    IOrderedContainer();

    void Add(IDrawable& item, int position, BoxPackingMode mode = BOX_SHRINK, bool is_visible = true);
    void SetVisibility(IDrawable& item, bool isVisible);

private:
    virtual void _Redraw(int w, int h) = 0;
    
protected:
    struct ContainerEntry{
        ContainerEntry(IDrawable& i) : item(i) {};
        int order; // Used to determine item's position within the container
        BoxPackingMode mode;
        mutable bool is_visible; //when false, item shouldn't be drawn
        mutable int size; // size in px for this item, used temporarily for calculations
        IDrawable& item;
    };
    struct ContainerEntryComparator{
        bool operator()(const ContainerEntry& x,const ContainerEntry& y) const {return x.order < y.order;}
    };
    
    /* This set contains all widgets contained within this container. */ //lol, this line CONTAINS such a cool sentence
    std::set< ContainerEntry, ContainerEntryComparator> children;
};

#endif // __IORDEREDCONTAINER_HPP__
