/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WButton.hpp"
#include "../CDisplay.hpp"
#include "allegro5/allegro.h"
#include "allegro5/allegro_primitives.h"
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>

WButton::WButton(std::string _text) : text(_text){
    SetForegroundColor(al_map_rgb(120, 140, 200));
    SetTextColor(al_map_rgb(255, 255, 255));
    event_clicked.Subscribe(this,&WButton::_ClickReaction);
    event_mouse_in.Subscribe(this,&WButton::_MouseInReaction);
    event_mouse_out.Subscribe(this,&WButton::_MouseOutReaction);
}

void WButton::SetForegroundColor(ALLEGRO_COLOR c){
    float r,g,b;
    al_unmap_rgb_f(c,&r,&g,&b);
    r *= 1.2f;
    g *= 1.2f;
    b *= 1.2f;
    
    foreground_color = c;
    
    pointed_color = al_map_rgb_f(r,g,b);
    color = foreground_color;
    
    SetNeedsRedrawing();
}

void WButton::SetTextColor(ALLEGRO_COLOR c){
    text_color = c;
}

void WButton::_Redraw(int w, int h){
    unsigned char r,g,b;
    al_unmap_rgb(color,&r,&g,&b);
    al_draw_filled_rectangle(0,0, w, h, color);
    al_draw_text(CDisplay::main_font, text_color, w/2, h/2 - 8, ALLEGRO_ALIGN_CENTRE, text.c_str());
}

void WButton::_ClickReaction(SCoordinates){
    event_activated.Happen();
}
void WButton::_MouseInReaction(){
    color = pointed_color;
    SetNeedsRedrawing();
}
void WButton::_MouseOutReaction(){
    color = foreground_color;
    SetNeedsRedrawing();
}

int WButton::_GetInnerMinimalHeight() {return 20;}
int WButton::_GetInnerMinimalWidth () {
    return al_get_text_width(CDisplay::main_font, text.c_str()) + 6; // 3px margin from both sides
}
