/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WEntry.hpp"
#include "../CDisplay.hpp"
#include "IContainer.hpp"
#include "allegro5/allegro.h"
#include "allegro5/allegro_primitives.h"
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <iostream>

WEntry::WEntry(std::string _text) : text(_text){
    event_clicked.Subscribe(this,&WEntry::_ClickReaction);
    event_keypress.Subscribe(this,&WEntry::_KeypressReaction);
}

void WEntry::SetText(std::string t){
    text = t;
    event_text_changed.Happen(text);
}
std::string WEntry::GetText(){ return text;}

void WEntry::_Redraw(int w, int h){
    al_draw_filled_rectangle(0,0, w, h, al_map_rgb(255,255,255));
    al_draw_text(CDisplay::main_font, al_map_rgb(0,0,0), 2, h/2 - 8, ALLEGRO_ALIGN_LEFT, text.c_str());
}

void WEntry::_ClickReaction(SCoordinates){
    // Focusing the WEntry
    IContainer* parent_container = dynamic_cast<IContainer*>(parent);
    if(parent_container != nullptr){
	parent_container->SetFocusedChild(this);
    }
}

void WEntry::_KeypressReaction(SKeypressData k){
    
    // Ignore control characters, and any non-ascii ones
    if(k.allegro_unicode > 31 && k.allegro_unicode < 127){
	text += (char)k.allegro_unicode;
	event_text_changed.Happen(text);
	SetNeedsRedrawing();
    }else if(k.allegro_unicode == 8){ //backspace
	if(text.size() != 0){
	    text.erase(text.size() - 1);
	    event_text_changed.Happen(text);
	    SetNeedsRedrawing();
	}
    }else if(k.allegro_unicode == 13){ //enter
	event_activated.Happen();
    }
    //std::cout << (int)k.allegro_unicode << std::endl;
}

int WEntry::_GetInnerMinimalHeight() {return 20;}
int WEntry::_GetInnerMinimalWidth () {
    return al_get_text_width(CDisplay::main_font, text.c_str()) + 6; // 3px margin from both sides
}
