/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WFixed.hpp"
#include "allegro5/allegro_primitives.h"

WFixed::WFixed(int w, int h) : _width(w), _height(h){
    event_clicked.Subscribe(this,&WFixed::_ClickReaction);
    event_mouse_out.Subscribe(this,&WFixed::_MouseOutReaction);
    event_mouse_over.Subscribe(this,&WFixed::_MouseOverReaction);
}

void WFixed::_Redraw(int w, int h){

    for(auto q : items){
        IDrawable* x = q.first;
        SCoordinates c = q.second;

        int width  = x->GetMinimalWidth ();
        int height = x->GetMinimalHeight();
        
        x->Draw(SDrawArea(c.x, c.y, width, height));
    }
}

void WFixed::AddItem(IDrawable& d, SCoordinates c){
    items[&d] = c;
    d.parent = this;
    
    SetNeedsRedrawing();
}

void WFixed::RemoveItem(IDrawable& d){
    items.erase(&d);
    d.parent = nullptr;
    SetNeedsRedrawing();
}

void WFixed::_ClickReaction(SCoordinates p){
    for(auto q : items){
        IDrawable* d = q.first;
        SCoordinates c = q.second;
        if(c.x <= p.x && c.y <= p.y && p.x < c.x + d->GetMinimalWidth() && p.y < c.y + d->GetMinimalHeight()){
            d->OnClicked(SCoordinates(p.x - c.x, p.y - c.y));
        }
    }
}

void WFixed::_MouseOverReaction(SCoordinates p){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto q : items) if(q.first->pointed) {previous = q.first; break;}
    // Find the currently pointed children
    IDrawable* current = nullptr;
    int dx, dy;
    for(auto q : items){
        IDrawable* d = q.first;
        SCoordinates c = q.second;
        if(c.x <= p.x && c.y <= p.y && p.x < c.x + d->GetMinimalWidth() && p.y < c.y + d->GetMinimalHeight()){
	    current = d;
	    dx = p.x - c.x;
	    dy = p.y - c.y;
	    break;
        }
    }
    
    if(previous != nullptr && current != previous){
	previous->OnMouseOut();
    }
    if(current != nullptr && current != previous){
	current->OnMouseIn();
    }
    if(current != nullptr){
	current->OnMouseMovedOver(SCoordinates(dx,dy));
    }
}
void WFixed::_MouseOutReaction(){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto q : items) if(q.first->pointed) {previous = q.first; break;}
    if(previous != nullptr){
	previous->OnMouseOut();
    }
}

int WFixed::_GetInnerMinimalHeight(){ return _height; }
int WFixed::_GetInnerMinimalWidth() { return _width;  }
