/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WLayeredView.hpp"

WLayeredView::WLayeredView() {
    event_clicked.Subscribe(this,&WLayeredView::_ClickReaction);
    event_mouse_out.Subscribe(this,&WLayeredView::_MouseOutReaction);
    event_mouse_over.Subscribe(this,&WLayeredView::_MouseOverReaction);
}

void WLayeredView::ActivateLayer(IDrawable& layer)
{
    //move layer to the front and make it visible
    for(auto i : children)
        if(&i.item == &layer)
        {
            children.erase(i);
            i.order = children.begin()->order - 1;
            i.is_visible = true;            
            children.insert(i);
            break;
        }
    SetNeedsRedrawing();
}

void WLayeredView::HideLayer(IDrawable& layer)
{
    SetVisibility(layer,false);
    
    //SetNeedsRedrawing(); // Unnecessary, because SetVisibility already sets the needs_redrawing flag
}
void WLayeredView::ShowLayer(IDrawable& layer)
{
    SetVisibility(layer,true);
    
    //SetNeedsRedrawing(); // Unnecessary, because SetVisibility already sets the needs_redrawing flag
}

void WLayeredView::SetOnlyVisibleLayer(IDrawable& layer)
{
    for(auto &i : children)
        if(&i.item == &layer)
            i.is_visible = true;            
        else
            i.is_visible = false;
    SetNeedsRedrawing();
}

void WLayeredView::_Redraw(int w, int h)
{
    for(auto i = children.rbegin() ; i != children.rend() ; i++)
        if(i->is_visible)
            i->item.Draw(SDrawArea(0,0,w,h));
}

void WLayeredView::_ClickReaction(SCoordinates p)
{
    //pass "click signal" to currently active layer
    for(auto i = children.begin() ; i != children.end() ; i++)
        if(i->is_visible)
        {
            i->item.OnClicked(p);
            break;
        }
}

void WLayeredView::_MouseOverReaction(SCoordinates p){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto i : children) if(i.item.pointed) {previous = &i.item; break;}
    // Find the currently pointed children
    IDrawable* current = nullptr;
    for(auto i : children){
        if(!i.is_visible) continue; // Skip invisible children
	current = &i.item;
	break;
    }
    
    if(previous != nullptr && current != previous){
	previous->OnMouseOut();
    }
    if(current != nullptr && current != previous){
	current->OnMouseIn();
    }
    if(current != nullptr){
	current->OnMouseMovedOver(SCoordinates(p.x,p.y));
    }
}

void WLayeredView::_MouseOutReaction(){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto i : children) if(i.item.pointed) {previous = &i.item; break;}
    if(previous != nullptr){
	previous->OnMouseOut();
    }
}

int WLayeredView::_GetInnerMinimalWidth()
{
    //find maximum of layers' widths
    int max = 0;
    for(auto i : children)
    {
        int w = i.item.GetMinimalWidth();
        if(w > max) max = w;
    }
    return max;
}

int WLayeredView::_GetInnerMinimalHeight()
{
    //find maximum of layers' heights
    int max = 0;
    for(auto i : children)
    {
        int h = i.item.GetMinimalHeight();
        if(h > max) max = h;
    }
    return max;
}
