/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WMultilineText.hpp"
#include "../CDisplay.hpp"

WMultilineText::WMultilineText(){
	SetTextColor(al_map_rgb(0,0,0));
}

WMultilineText::WMultilineText(std::string text){
	SetTextColor(al_map_rgb(0,0,0));
	Append(text);
}

void WMultilineText::Append(std::string text){
	std::vector<std::string> t = _Split(text);
	_text.insert(_text.end(),t.begin(),t.end());
	SetNeedsRedrawing();
}

void WMultilineText::SetText(std::string text){
	_text.clear();
	Append(text);
}

void WMultilineText::SetTextColor(ALLEGRO_COLOR c){ _text_color = c; }

void WMultilineText::SetSpacing(int s){
	_spacing = s;
	SetNeedsRedrawing();
}

std::vector<std::string> WMultilineText::_Split(std::string str){
    std::vector<std::string> res;

    size_t pos = 0;
    std::string line;
    while ((pos = str.find('\n')) != std::string::npos) {
        line = str.substr(0, pos);
        res.push_back(line);
        str.erase(0, pos + 1);
    }
    res.push_back(str);
    return res;
}

void WMultilineText::_Redraw(int w, int h){
	int x = _spacing/2;
	for(std::string s : _text){
		al_draw_text(CDisplay::main_font, _text_color, 0 , x , 0, s.c_str());
		//   \/ font size
		x += 12 + _spacing;
	}
}

int WMultilineText::_GetInnerMinimalWidth(){
	//maximum of all lines
	int max = 0;
	for(std::string s : _text){
		int curr = al_get_text_width(CDisplay::main_font, s.c_str());
		if (curr > max) max = curr;
	}
	return max;
}
int WMultilineText::_GetInnerMinimalHeight(){
	//sum of all lines + spacing
	if(_text.size() == 0) return 0;
	//       font size \/
	return _text.size()*12 + _text.size()*_spacing;
}
