/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WProportional.hpp"

WProportional::WProportional(double horizontal_hook_position_on_parent,
							 double vertical_hook_position_on_parent,
							 double horizontal_hook_position_on_child,
							 double vertical_hook_position_on_child,
							 double horizontal_size_fraction,
							 double vertical_size_fraction) :			 
			 _horizontal_hook_position_on_parent(horizontal_hook_position_on_parent),
			 _vertical_hook_position_on_parent(vertical_hook_position_on_parent),
			 _horizontal_hook_position_on_child(horizontal_hook_position_on_child),
			 _vertical_hook_position_on_child(vertical_hook_position_on_child),
			 _horizontal_size_fraction(horizontal_size_fraction),
			 _vertical_size_fraction(vertical_size_fraction)
{	
    event_clicked.Subscribe(this,&WProportional::_ClickReaction);
    event_mouse_out.Subscribe(this,&WProportional::_MouseOutReaction);
    event_mouse_over.Subscribe(this,&WProportional::_MouseOverReaction);
}

void WProportional::Add(IDrawable* i){
	if(i->parent != nullptr) return;
	child = i;
	child->parent = this;
}

void WProportional::_Redraw(int parent_w, int parent_h){
	if(child == nullptr) return;
	_child_drawn_width  = parent_w * _horizontal_size_fraction;
	_child_drawn_height = parent_h * _vertical_size_fraction;
	int child_min_w = child->GetMinimalWidth();
	int child_min_h = child->GetMinimalHeight();
	if(_horizontal_size_fraction == 0.0 || _child_drawn_width  < child_min_w) _child_drawn_width  = child_min_w;
	if(  _vertical_size_fraction == 0.0 || _child_drawn_height < child_min_h) _child_drawn_height = child_min_h;
	_child_drawn_x = _horizontal_hook_position_on_parent*parent_w - _child_drawn_width*_horizontal_hook_position_on_child;
	_child_drawn_y =   _vertical_hook_position_on_parent*parent_h - _child_drawn_height*_vertical_hook_position_on_child;
	
	child->Draw(SDrawArea(_child_drawn_x, _child_drawn_y ,_child_drawn_width, _child_drawn_height));
}

int WProportional::_GetInnerMinimalHeight(){
	if(child == nullptr) return 0;
	return child->GetMinimalHeight();
}
int WProportional::_GetInnerMinimalWidth(){
	if(child == nullptr) return 0;
	return child->GetMinimalWidth();
}

void WProportional::_ClickReaction(SCoordinates p){
	if(child == nullptr) return;
	if(p.x >= _child_drawn_x && p.y >= _child_drawn_y && p.x <= _child_drawn_x + _child_drawn_width && p.y <= _child_drawn_y + _child_drawn_height)
		child->OnClicked(SCoordinates(p.x - _child_drawn_x, p.y - _child_drawn_y));
}

void WProportional::_MouseOverReaction(SCoordinates p){
	if(child == nullptr) return;
	if(p.x >= _child_drawn_x && p.y >= _child_drawn_y && p.x <= _child_drawn_x + _child_drawn_width && p.y <= _child_drawn_y + _child_drawn_height){
		if(child->pointed == false) child->OnMouseIn();
		child->OnMouseMovedOver(SCoordinates(p.x - _child_drawn_x, p.y - _child_drawn_y));
	}else{
		if(child->pointed == true) child->OnMouseOut();
	}
}
void WProportional::_MouseOutReaction(){
	if(child == nullptr) return;
	if(child->pointed == true) child->OnMouseOut();
}
