#ifndef __WPROPORTIONAL_HPP__
#define __WPROPORTIONAL_HPP__

/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "IContainer.hpp"

class WProportional : public IContainer{
public:
	/* This widget contains one (1) widget, and manages it's position.
	 * It works this way:
	 * There are two 'hooks' defined both in the body of this widget's
	 * and the child's (apply the child with SetChild(...)). Neither of
	 * hooks' position is an offset from a widget's corner, both are a
	 * fraction of size of the widget. So a hook on (0.5, 0.5) is an
	 * ideally centered one.
	 * WProportional takes care so that both these hooks are on exactly
	 * the same place on the display, it draws the child widget on such
	 * position, so that the two hooks are perfectly alligned.
	 * The size fraction variables may be used to resize the inned
	 * widget. If set to 0, the child will always be shrinked to its preferred
	 * minimum. In any other case, the width/height of parent are
	 * multiplied by the scale factors and the output will be used
	 * as child's drawn size.*/
	WProportional(double horizontal_hook_position_on_parent,
				  double vertical_hook_position_on_parent,
				  double horizontal_hook_position_on_child,
				  double vertical_hook_position_on_child,
				  double horizontal_size_fraction = 0.0,
				  double vertical_size_fraction = 0.0);
	void Add(IDrawable* item);
private:
	IDrawable* child = nullptr;
	
	double _horizontal_hook_position_on_parent;
	double _vertical_hook_position_on_parent;
	double _horizontal_hook_position_on_child;
	double _vertical_hook_position_on_child;
	double _horizontal_size_fraction;
	double _vertical_size_fraction;
	
	int _child_drawn_x;
	int _child_drawn_y;
	int _child_drawn_width;
	int _child_drawn_height;
	
    virtual void _Redraw(int w, int h) override;
    
    void _ClickReaction(SCoordinates p);
    void _MouseOverReaction(SCoordinates p);
    void _MouseOutReaction();
    
    virtual int _GetInnerMinimalHeight() override;
    virtual int _GetInnerMinimalWidth () override;
};

#endif
