/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "WVBox.hpp"
#include <allegro5/allegro_primitives.h>

WVBox::WVBox() {
    event_clicked.Subscribe(this,&WVBox::_ClickReaction);
    event_mouse_out.Subscribe(this,&WVBox::_MouseOutReaction);
    event_mouse_over.Subscribe(this,&WVBox::_MouseOverReaction);
}

void WVBox::_Redraw(int w, int h){
    
    _RecalculateSizes(h, &IDrawable::GetMinimalHeight);
    
    int x=0, y=0;
    
    for(auto i : children){
        if(!i.is_visible) continue; //an unvisible widget. skip it...
        
        int width = w;
        int height = i.size;
        
        i.item.Draw(SDrawArea(x,y,width,height));
        
        y += i.size;
        y += _spacing;
    }
    
    // Remember my size (for click recognition)
    _current_height = h;
    _current_width  = w;
    
}

int WVBox::_GetInnerMinimalHeight() {
    // Sum of all items' heights and gaps
    int sum = 0;
    for(auto i : children){
        sum += i.item.GetMinimalHeight();
        sum += _spacing;
    }
    if(sum >= 0) sum -= _spacing;
    return sum;
}
int WVBox::_GetInnerMinimalWidth () {
    // Maximum of all items' widths
    int max = 0;
    for(auto i : children){
        int w = i.item.GetMinimalWidth();
        if (w > max) max = w;
    }
    return max;
}

void WVBox::_ClickReaction(SCoordinates p){
    if(_current_height < p.y || _current_width < p.x) return;
    int sum=0;
    for(auto i : children){
        if(!i.is_visible) continue;
        if(p.y < sum + i.size) { // This item was clicked!
            i.item.OnClicked(SCoordinates(p.x, p.y - sum));
            return;
        }
        sum += i.size;
        sum += _spacing;
        if(p.y < sum){ // Empty space between items was clicked
            return;
        }
    }
}

void WVBox::_MouseOverReaction(SCoordinates p){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto q : children) if(q.item.pointed) {previous = &q.item; break;}
    // Find the currently pointed children
    IDrawable* current = nullptr;
    int sum = 0;
    int dx, dy;
    for(auto i : children){
        if(!i.is_visible) continue;
        if(p.y < sum + i.size) { // This item was moved-over!
	    current = &i.item;
	    dx = p.x;
	    dy = p.y - sum;
	    break;
        }
        sum += i.size;
        sum += _spacing;
        if(p.y < sum){ // Empty space between items was clicked
            current = nullptr;
	    break;
        }
    }
    
    if(previous != nullptr && current != previous){
	previous->OnMouseOut();
    }
    if(current != nullptr && current != previous){
	current->OnMouseIn();
    }
    if(current != nullptr){
	current->OnMouseMovedOver(SCoordinates(dx,dy));
    }
}
void WVBox::_MouseOutReaction(){
    // Find the previously pointed children
    IDrawable* previous = nullptr;
    for(auto q : children) if(q.item.pointed) {previous = &q.item; break;}
    if(previous != nullptr){
	previous->OnMouseOut();
    }
}
